from flask import Flask, render_template, jsonify, request
from copy import copy
app = Flask(__name__)

# Vitals
vitalsHeartrate = -1
vitalsOxygen = -1
vitalsBodytemp = -1.0

@app.route('/api/vitals/heartrate')
def getVitalsHeartrate():
	returnArr = [ { 'heartrate': vitalsHeartrate } ]
	return jsonify(returnArr)

@app.route('/api/vitals/heartrate', methods=['POST'])
def setVitalsHeartrate():
	global vitalsHeartrate
	json = request.get_json()
	try:
		vitalsHeartrate = json['heartrate']
	except:
		return 'Incorrect usage.\nUsage: { heartrate: INT }\n', 400
	return 'Information set successfully', 204

@app.route('/api/vitals/oxygen')
def getVitalsOxygen():
	returnArr = [ { 'oxygen': vitalsOxygen } ]
	return jsonify(returnArr)

@app.route('/api/vitals/oxygen', methods=['POST'])
def setVitalsOxygen():
	global vitalsOxygen
	json = request.get_json()
	try:
		vitalsOxygen = json['oxygen']
	except:
		return 'Incorrect usage.\nUsage: { oxygen: INT }\n', 400
	return 'Information set successfully', 204

@app.route('/api/vitals/bodytemp')
def getVitalsBodytemp():
	returnArr = [ { 'bodytemp': vitalsBodytemp } ]
	return jsonify(returnArr)

@app.route('/api/vitals/bodytemp', methods=['POST'])
def setVitalsBodytemp():
	global vitalsBodytemp
	json = request.get_json()
	try:
		vitalsBodytemp = json['bodytemp']
	except:
		return 'Incorrect usage.\nUsage: { bodytemp: FLOAT }\n', 400
	return 'Information set successfully', 204

@app.route('/api/vitals')
def getVitals():
	returnArr = [ { 'heartrate': vitalsHeartrate, 'oxygen': vitalsOxygen, 'bodytemp': vitalsBodytemp } ]
	return jsonify(returnArr)

@app.route('/api/vitals', methods=['POST'])
def setVitals():
	global vitalsHeartrate
	global vitalsOxygen
	global vitalsBodytemp

	json = request.get_json()
	try:
		# This is a bit ugly but its just how I'm checking that everything is there without setting variables if the json is incorrect
		tempH = json['heartrate'] 
		tempO = json['oxygen']
		tempB = json['bodytemp']
		vitalsHeartrate = tempH
		vitalsOxygen = tempO
		vitalsBodytemp = tempB
	except:
		return 'Incorrect usage.\nUsage: { heartrate: INT, oxygen: INT, bodytemp: FLOAT }\n', 400
	return 'Information set successfully', 204

# Fitness
fitnessSteps = -1

@app.route('/api/fitness/steps')
def getSteps():
	return getFitness() # This is the same for now.

@app.route('/api/fitness/steps', methods=['POST'])
def setFitnessSteps():
	global fitnessSteps
	json = request.get_json()
	try:
		vitalsBodytemp = json['steps']
	except:
		return 'Incorrect usage.\nUsage: { steps: INT }\n', 400
	return 'Information set successfully', 204


@app.route('/api/fitness')
def getFitness():
	returnArr = [ { 'steps': fitnessSteps } ]
	return jsonify(returnArr)

@app.route('/api/fitness', methods=['POST'])
def setFitness():
	global fitnessSteps
	json = request.get_json()
	try:
		vitalsBodytemp = json['steps']
	except:
		return 'Incorrect usage.\nUsage: { steps: INT }\n', 400
	return 'Information set successfully', 204


# Cyberware management
currentId = 1
cyberware = []
newCyberwareTemplate = { "id": -1, "name": "", "hotpluggable": False, "lastMalfunction": "", "frontendAwknowledge": False }


# VERY descriptive variable names, huh?
#cyberwareIdIndex = 0
#cyberwareNameIndex = 1
#cyberwareHotpluggableIndex = 2
#cyberwareLastMalfunction = 3
#cyberwareFrontwareAwknowledge = 4

# This makes the system aware of a new piece of hardware.
# While, for the most part, not required due to the design of this project,
#  it's handy for error reporting and communication between hardware and
#  the end user.
#
# Arguments: { name: STRING, hotpluggable: BOOL }
# Returns:   { id: INT }
# name: A human-readable name
# hotpluggable argument: Used for removable modules known as shards.
#
# id: Returns an id of the hardware.
@app.route('/api/cyberware/add', methods=['POST'])
def addCyberware():
	global cyberware
	global currentId
	json = request.get_json()

	try:
		tempName = json['name']
		tempHotpluggable = json['hotpluggable']
		tempNewCyberware = copy(newCyberwareTemplate)

		tempNewCyberware['id'] = currentId
		tempNewCyberware['name'] = tempName
		tempNewCyberware['hotpluggable'] = tempHotpluggable

		currentId = currentId + 1

		cyberware.append(tempNewCyberware)
	except:
		return 'Incorrect usage.\nUsage: { name: STRING, hotpluggable: BOOL }\n', 400
	return jsonify([ { "id": currentId-1 } ]), 204

# Arguments: { id: INT }
@app.route('/api/cyberware/remove', methods=['POST'])
def removeCyberware():
	global cyberware
	json = request.get_json()

	try:
		desiredId = json['id']
		i = 0
		for c in cyberware:
			if c['id'] == desiredId:
				del cyberware[i]
				return 'Cyberware removed', 204
			i = i + 1
	except:
		return 'Incorrect usage.\nUsage: { id: INT }\n', 400
	return 'ID Invalid\n', 400

# Returns: { id: INT, name: STRING, hotpluggable: BOOL, lastMalfunction: STRING, frontendAwknowledge: BOOL }
# id:                 Unique identifier of the hardware
# name:               Human-readable name
# hotpluggable:       Hardware can be removed during runtime.
# lastMalfunction:    A string with information on the last malfunction.
# frontendAwknowlege: A tool for the frontend to keep track of whether it's aware of the hardware.
@app.route('/api/cyberware')
def getCyberware():
	return jsonify(cyberware)

# Arguments { id: INT }
@app.route('/api/cyberware/reset', methods=['POST'])
def resetCyberwareMalfunction():
	return 'Not Implemented', 501


@app.route('/')
def uiindex():
	return render_template('index.html')
